## Dose response MBMA example

rm(list = ls())
gc()

modelName <- "doseResponseMBMA1"

## Relative paths assuming the working directory is the script directory
## containing this script
scriptDir <- getwd()
projectDir <- dirname(scriptDir)
figDir <- file.path(projectDir, "deliv", "figure", modelName)
tabDir <- file.path(projectDir, "deliv", "table", modelName)
dataDir <- file.path(projectDir, "data", "derived")
modelDir <- file.path(projectDir, "model")
outDir <- file.path(modelDir, modelName)
toolsDir <- file.path(scriptDir, "tools")

source(file.path(scriptDir, "pkgSetup.R"))

library(rstan)
library(bayesplot)
library(tidyverse)
library(parallel)
source(file.path(toolsDir, "stanTools.R"))

rstan_options(auto_write = TRUE)
options(mc.cores = parallel::detectCores())

set.seed(10271998) ## not required but assures repeatable results

################################################################################################

## get data file
xdata <- read.csv(file.path(dataDir, "doseResponse.csv"))

## create data set
data <- with(xdata,
             list(
                 nStudies = max(study),
                 nArms = nrow(xdata),
                 study = study,
                 dose = dose,
                 n = n,
                 response = resp))

## create initial estimates
init <- function() list(
    e0Hat = exp(rnorm(1, log(2), 0.5)),
    emaxHat = exp(rnorm(1, log(10), 0.5)),
    ed50 = exp(rnorm(1, log(5), 0.5)),
    sigma = exp(rnorm(1, log(0.5), 1)),
    omegaE0 = exp(rnorm(1, log(0.2), 1)),
    omegaEmax = exp(rnorm(1, log(0.2), 1)),
    e0 = rep(2, 5),
    emax = rep(10, 5))

## Specify the variables for which you want history and density plots
parametersToPlot <- c("e0Hat", "emaxHat", "ed50", "sigma", "omegaE0", "omegaEmax")

## Additional variables to monitor
otherRVs <- c("e0", "emax", "responseCond", "responsePred")

parameters <- c(parametersToPlot, otherRVs)

################################################################################################
# run Stan

nChains <- 4
nPost <- 1000 ## Number of post-burn-in samples per chain after thinning
nBurn <- 1000 ## Number of burn-in samples per chain after thinning
nThin <- 1

nIter <- (nPost + nBurn) * nThin
nBurnin <- nBurn * nThin

dir.create(outDir)

fit <- stan(file = file.path(modelDir, paste(modelName, ".stan", sep = "")),
            data = data,
            pars = parameters,
            iter = nIter,
            warmup = nBurnin,
            thin = nThin, 
            init = init,
            control = list(stepsize = 0.01, adapt_delta = 0.99),
            chains = nChains,
            cores = min(nChains, parallel::detectCores())) ## Run chains in parallel

save(fit, file = file.path(outDir, paste(modelName, "Fit.Rsave", sep = "")))
##load(file.path(outDir, paste(modelName, "Fit.Rsave", sep = "")))

################################################################################################
## posterior distributions of parameters

dir.create(figDir)
dir.create(tabDir)

## open graphics device
pdf(file = file.path(figDir, paste(modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

options(bayesplot.base_size = 12,
        bayesplot.base_family = "sans")
color_scheme_set(scheme = "brightblue")
myTheme <- theme(text = element_text(size = 12), axis.text = element_text(size = 12))

rhats <- rhat(fit, pars = parametersToPlot)
mcmc_rhat(rhats) + yaxis_text() + myTheme

ratios1 <- neff_ratio(fit, pars = parametersToPlot)
mcmc_neff(ratios1) + yaxis_text() + myTheme

posterior <- as.array(fit)
mcmc_acf(posterior, pars = parametersToPlot) + myTheme

mcmcHistory(fit, pars = parametersToPlot, nParPerPage = 4, myTheme = myTheme)

mcmc_dens_overlay(posterior, parametersToPlot)
mcmc_dens(posterior, parametersToPlot) + myTheme

pairs(fit, pars = parametersToPlot)

ptable <- monitor(as.array(fit, pars = parametersToPlot), warmup = 0, print = FALSE)
write.csv(ptable, file = file.path(tabDir, paste(modelName, "ParameterTable.csv", sep = "")))

################################################################################################
## posterior predictive distributions

# prediction of future observations in the same studies, i.e., posterior predictions
# conditioned on observed data from the same study

pred <- as.data.frame(fit, pars = "responseCond") %>%
  gather(factor_key = TRUE) %>%
      group_by(key) %>%
            summarize(lb = quantile(value, probs = 0.05),
                      median = quantile(value, probs = 0.5),
                      ub = quantile(value, probs = 0.95)) %>%
                          bind_cols(xdata)

p1 <- ggplot(pred, aes(x = dose, y = resp))
p1 <- p1 + geom_point() +
    labs(title = "individual predictions", x = "dose", y = "response") +
        theme(text = element_text(size = 12), axis.text = element_text(size = 12),
              legend.position = "none", strip.text = element_text(size = 8)) +
                  facet_wrap(~ study)
p1 + geom_line(aes(x = dose, y = median)) +
    geom_ribbon(aes(ymin = lb, ymax = ub), alpha = 0.25)

# prediction of future observations in a new study, i.e., posterior predictive distributions

pred <- as.data.frame(fit, pars = "responsePred") %>%
  gather(factor_key = TRUE) %>%
        group_by(key) %>%
            summarize(lb = quantile(value, probs = 0.05),
                      median = quantile(value, probs = 0.5),
                      ub = quantile(value, probs = 0.95)) %>%
                          bind_cols(xdata)

p1 <- ggplot(pred, aes(x = dose, y = resp))
p1 <- p1 + geom_point() +
    labs(title = "population predictions", x = "dose", y = "response") +
        theme(text = element_text(size = 12), axis.text = element_text(size = 12),
              legend.position = "none", strip.text = element_text(size = 8)) +
                  facet_wrap(~ study)
p1 + geom_line(aes(x = dose, y = median)) +
    geom_ribbon(aes(ymin = lb, ymax = ub), alpha = 0.25)

dev.off()
