rm(list = ls())
gc()

modelName <- "singleDosePK2"

## Relative paths assuming the working directory is the script directory
## containing this script
scriptDir <- getwd()
projectDir <- dirname(scriptDir)
figDir <- file.path(projectDir, "deliv", "figure", modelName)
tabDir <- file.path(projectDir, "deliv", "table", modelName)
dataDir <- file.path(projectDir, "data", "derived")
modelDir <- file.path(projectDir, "model")
outDir <- file.path(modelDir, modelName)
toolsDir <- file.path(scriptDir, "tools")

source(file.path(scriptDir, "pkgSetup.R"))

library(rstan)
library(bayesplot)
library(tidyverse)
library(parallel)
source(file.path(toolsDir, "stanTools.R"))

rstan_options(auto_write = TRUE)
options(mc.cores = parallel::detectCores())

set.seed(10271998) ## not required but assures repeatable results

################################################################################################

## get data file
xdata <- read.csv(file.path(dataDir, "fxa.data.csv"))
xdata <- subset(xdata, time > 0 & dose != 0)

nObs <- nrow(xdata)
start <- (1:nObs)[!duplicated(xdata$subject)]
end <- c(start[-1] - 1, nObs)

## create data set
data <- with(xdata,
             list(
                 nSubjects = max(subject),
                 nObs = nObs,
                 dose = 1000 * dose[!duplicated(subject)],
                 weight = weight[!duplicated(subject)],
                 start = start,
                 end = end,
                 time = time,
                 cObs = cobs))

## create initial estimates
init <- function(){
  list(CLHat = exp(rnorm(1, log(10), 0.2)),
       QHat = exp(rnorm(1, log(15), 0.2)),
       V1Hat = exp(rnorm(1, log(35), 0.2)),
       V2Hat = exp(rnorm(1, log(105), 0.2)),
       kaHat = exp(rnorm(1, log(2), 0.2)),
       omega = exp(rnorm(5, log(0.25), 0.5)),
       rho = diag(5),
       sigma = runif(1, 0.5, 2),
       logtheta = matrix(rep(log(c(10, 15, 35, 105, 2)), 
                             ea = data$nSubjects), 
                         nrow = data$nSubjects))
}

## Specify the variables for which you want history and density plots
parametersToPlot <- c("CLHat", "QHat", "V1Hat", "V2Hat", "kaHat",
                      "sigma", "omega", "rho")

## Additional variables to monitor
otherRVs <- c("cObsCond", "cObsPred", "CL", "Q", "V1", "V2", "ka")

parameters <- c(parametersToPlot, otherRVs)

################################################################################################
# run Stan

nChains <- 4
nPost <- 500 ## Number of post-burn-in samples per chain after thinning
nBurn <- 500 ## Number of burn-in samples per chain after thinning
nThin <- 1

nIter <- (nPost + nBurn) * nThin
nBurnin <- nBurn * nThin

fit <- stan(file = file.path(modelDir, paste(modelName, ".stan", sep = "")),
            data = data,
            pars = parameters,
            iter = nIter,
            warmup = nBurnin,
            thin = nThin, 
            init = init,
            chains = nChains,
            control = list(adapt_delta = 0.9))

dir.create(outDir)
save(fit, file = file.path(outDir, paste(modelName, "Fit.Rsave", sep = "")))
##load(file.path(outDir, paste(modelName, "Fit.Rsave", sep = "")))

################################################################################################
## posterior distributions of parameters
 
dir.create(figDir)
dir.create(tabDir)

## open graphics device
pdf(file = file.path(figDir, paste(modelName,"Plots%03d.pdf", sep = "")),
	width = 6, height = 6, onefile = F)

## Remove diagonal & redundant elements of rho
dimRho <- nrow(init()$rho)
parametersToPlot <- c(parametersToPlot,
                     paste("rho[", matrix(apply(expand.grid(1:dimRho, 1:dimRho), 1, paste, collapse = ","),
                                          ncol = dimRho)[upper.tri(diag(dimRho), diag = FALSE)], "]", sep = ""))
parametersToPlot <- setdiff(parametersToPlot, "rho")

options(bayesplot.base_size = 12,
        bayesplot.base_family = "sans")
color_scheme_set(scheme = "brightblue")
myTheme <- theme(text = element_text(size = 12), axis.text = element_text(size = 12))

rhats <- rhat(fit, pars = parametersToPlot)
mcmc_rhat(rhats) + yaxis_text() + myTheme

ratios1 <- neff_ratio(fit, pars = parametersToPlot)
mcmc_neff(ratios1) + yaxis_text() + myTheme

mcmcHistory(fit, pars = parametersToPlot, nParPerPage = 5, myTheme = myTheme)
mcmcDensity(fit, pars = parametersToPlot, nParPerPage = 16, byChain = TRUE, 
            myTheme = theme(text = element_text(size = 12), axis.text = element_text(size = 10)))
mcmcDensity(fit, pars = parametersToPlot, nParPerPage = 16, 
            myTheme = theme(text = element_text(size = 12), axis.text = element_text(size = 10)))

pairs(fit, pars = parametersToPlot[!grepl("rho", parametersToPlot)])

ptable <- monitor(as.array(fit, pars = parametersToPlot), warmup = 0, print = FALSE)
write.csv(ptable, file = file.path(tabDir, paste(modelName, "ParameterTable.csv", sep = "")))

################################################################################################
## posterior predictive distributions

# prediction of future observations in the same studies, i.e., posterior predictions
# conditioned on observed data from the same study

pred <- as.data.frame(fit, pars = "cObsCond") %>%
    gather(factor_key = TRUE) %>%
        mutate(value = ifelse(value == -99, NA, value)) %>%
        group_by(key) %>%
            summarize(lb = quantile(value, probs = 0.05, na.rm = TRUE),
                      median = quantile(value, probs = 0.5, na.rm = TRUE),
                      ub = quantile(value, probs = 0.95, na.rm = TRUE)) %>%
                          bind_cols(xdata)

doses <- sort(unique(xdata$dose))
for(thisDose in doses){
    plotdata <- subset(pred, dose == thisDose)
    p1 <- ggplot(plotdata, aes(x = time, y = cobs))
    p1 <- p1 + geom_point() +
        labs(title = paste(thisDose, "mg\n individual predictions"),
             x = "time (h)",
             y = "ME-2 plasma concentration (ng/mL)") +
                 theme(text = element_text(size = 12), axis.text = element_text(size = 12),
                       legend.position = "none", strip.text = element_text(size = 8)) +
                           facet_wrap(~ subject)
    print(p1 + geom_line(aes(x = time, y = median)) +
        geom_ribbon(aes(ymin = lb, ymax = ub), alpha = 0.25))
}

# prediction of future observations in a new study, i.e., posterior predictive distributions

pred <- as.data.frame(fit, pars = "cObsPred") %>%
    gather(factor_key = TRUE) %>%
        mutate(value = ifelse(value == -99, NA, value)) %>%
        group_by(key) %>%
            summarize(lb = quantile(value, probs = 0.05, na.rm = TRUE),
                      median = quantile(value, probs = 0.5, na.rm = TRUE),
                      ub = quantile(value, probs = 0.95, na.rm = TRUE)) %>%
                          bind_cols(xdata)

doses <- sort(unique(xdata$dose))
for(thisDose in doses){
    plotdata <- subset(pred, dose == thisDose)
    p1 <- ggplot(plotdata, aes(x = time, y = cobs))
    p1 <- p1 + geom_point() +
        labs(title = paste(thisDose, "mg\n population predictions"),
             x = "time (h)",
             y = "ME-2 plasma concentration (ng/mL)") +
                 theme(text = element_text(size = 12), axis.text = element_text(size = 12),
                       legend.position = "none", strip.text = element_text(size = 8)) +
                           facet_wrap(~ subject)
    print(p1 + geom_line(aes(x = time, y = median)) +
        geom_ribbon(aes(ymin = lb, ymax = ub), alpha = 0.25))
}

dev.off()
